/*
 * DirectedGraph.h
 *
 *   Created on: Jun 19, 2014
 *       Author: David Rosca
 *  Modified by: Jan Broz
 */

#ifndef DIRECTED_GRAPH_H_
#define DIRECTED_GRAPH_H_

#include <unordered_map>
#include <vector>
#include <set>
#include <tuple>

#include "../GraphBase.h"
#include "../GraphTypes.h"
#include "../common/Macros.h"
#include "../common/Node.h"
#include "DirectedEdge.h"

namespace graph {

class UndirectedGraph;

class DirectedGraph : public GraphBase {
	ext::unordered_map< Node, int > nodeValues;
	ext::unordered_map< DirectedEdge, int > edgeValues;
	ext::unordered_map< Node, ext::vector< Node > > m_embedding;

public:
	DirectedGraph();

	virtual GRAPH_TYPE getType() const override;
	virtual REPRESENTATION getRepresentation() const override = 0 ;

	/// constructs an directed graph from any representation of other directed graph
	void fromDirected( const DirectedGraph & orig );
	/// converts undirected graph into directed by making 2 directed edges in both directions from each undirected one
	void fromUndirected( const UndirectedGraph & orig );


	// interface for all graph reprezentations

	virtual ext::set< Node > getNodes() const = 0;
	virtual ext::set< DirectedEdge > getEdges() const = 0;

	virtual size_t nodeCount() const = 0;
	virtual size_t edgeCount() const = 0;

	virtual bool hasNode( const Node & node ) const = 0;
	virtual bool hasEdge( const DirectedEdge & edge ) const = 0;

	virtual bool hasEdge( const Node & from, const Node & to ) const = 0;
	virtual ext::set< DirectedEdge > findEdges( const Node & from, const Node & to ) const = 0;

	// !!todo!! remove and replace with successors and predecessors
	/// deprecated! use successors or predcessors instead
	virtual ext::set< Node > neighbors( const Node & node ) const = 0;
	/// deprecated! use successorEdges or predcessorEdges instead
	virtual ext::set< DirectedEdge > neighborEdges( const Node & node ) const = 0;

	virtual ext::set< Node > successors( const Node & node ) const = 0;
	virtual ext::set< Node > predecessors( const Node & node ) const = 0;
	virtual ext::set< DirectedEdge > successorEdges( const Node & node ) const = 0;
	virtual ext::set< DirectedEdge > predecessorEdges( const Node & node ) const = 0;

	virtual bool addNode( const Node & node ) = 0;
	// these 2 methods must be called from the implementation in subclasses
	virtual bool addNode( const Node & node, int value ) = 0;
	virtual bool removeNode( const Node & node ) = 0;

	virtual bool addEdge( const DirectedEdge & edge ) = 0;
	// these 2 methods must be called from the implementation in subclasses
	virtual bool addEdge( const DirectedEdge & edge, int value ) = 0;
	virtual bool removeEdge( const DirectedEdge & edge ) = 0;


	// weight of nodes and edges

	bool areNodesWeighted() const;
	bool areEdgesWeighted() const;

	int getNodeValue( const Node & node ) const;
	void setNodeValue( const Node & node, int value );

	int getEdgeValue( const DirectedEdge & edge ) const;
	void setEdgeValue( const DirectedEdge & edge, int value );

	ext::unordered_map< Node, ext::vector< Node > > getEmbedding() const;
	void setEmbedding( const ext::unordered_map< Node, ext::vector< Node > > embedding );


	// commmon methods

	int compare( const ObjectBase & other ) const override {
		if (ext::type_index( typeid( *this ) ) == ext::type_index( typeid( other ) ))
			return this->compare( (decltype( *this ))other );
		return ext::type_index( typeid( *this ) ) - ext::type_index( typeid( other ) );
	}
	int compare( const DirectedGraph & other ) const;

	virtual void operator>>( std::ostream & out ) const override;
	virtual explicit operator std::string() const override;
	virtual std::string toString() const override;

	//static DirectedGraph * parse( ext::deque< sax::Token >::iterator & input );
	//void compose( ext::deque< sax::Token > & out ) const override;

	/*static const std::string & getXmlTagName() {
		static std::string xmlTagName = "DirectedGraph";

		return xmlTagName;
	}*/

	friend class GraphToXMLComposer;
	friend class GraphToStringComposer;

};

} // namespace graph

#endif // DIRECTED_GRAPH_H_
