/*
 * SuffixTrieTerminatingSymbol.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#ifndef SUFFIX_TRIE_TERMINATING_SYMBOL_H_
#define SUFFIX_TRIE_TERMINATING_SYMBOL_H_

#include <vector>
#include <list>
#include <string>
#include <set>
#include <core/components.hpp>
#include "SuffixTrieNodeTerminatingSymbol.h"
#include <object/ObjectBase.h>

namespace indexes {

class GeneralAlphabet;
class TerminatingSymbol;

/**
 * Represents regular expression parsed from the XML. Regular expression is stored
 * as a tree of RegExpElement.
 */
class SuffixTrieTerminatingSymbol final : public alib::ObjectBase, public alib::Components < SuffixTrieTerminatingSymbol, DefaultSymbolType, ext::tuple < GeneralAlphabet >, ext::tuple < TerminatingSymbol > > {
protected:
	SuffixTrieNodeTerminatingSymbol * m_tree;

public:
	/**
	 * @copydoc SuffixTrieNodeTerminatingSymbol::clone() const
	 */
	virtual ObjectBase * clone ( ) const;

	/**
	 * @copydoc SuffixTrieNodeTerminatingSymbol::plunder() const
	 */
	virtual ObjectBase * plunder ( ) &&;

	explicit SuffixTrieTerminatingSymbol ( ext::set < DefaultSymbolType > alphabet, DefaultSymbolType terminatingSymbol );
	explicit SuffixTrieTerminatingSymbol ( ext::set < DefaultSymbolType > alphabet, DefaultSymbolType terminatingSymbol, SuffixTrieNodeTerminatingSymbol tree );
	explicit SuffixTrieTerminatingSymbol ( DefaultSymbolType terminatingSymbol, SuffixTrieNodeTerminatingSymbol tree );

	/**
	 * Copy constructor.
	 * @param other tree to copy
	 */
	SuffixTrieTerminatingSymbol ( const SuffixTrieTerminatingSymbol & other );
	SuffixTrieTerminatingSymbol ( SuffixTrieTerminatingSymbol && other ) noexcept;
	SuffixTrieTerminatingSymbol & operator =( const SuffixTrieTerminatingSymbol & other );
	SuffixTrieTerminatingSymbol & operator =( SuffixTrieTerminatingSymbol && other ) noexcept;
	~SuffixTrieTerminatingSymbol ( ) noexcept;

	/**
	 * @return Root node of the regular expression tree
	 */
	const SuffixTrieNodeTerminatingSymbol & getRoot ( ) const;

	/**
	 * @return Root node of the regular expression tree
	 */
	SuffixTrieNodeTerminatingSymbol & getRoot ( );

	/**
	 * Sets the root node of the regular expression tree
	 * @param tree root node to set
	 */
	void setTree ( SuffixTrieNodeTerminatingSymbol tree );

	const ext::set < DefaultSymbolType > & getAlphabet ( ) const {
		return accessComponent < GeneralAlphabet > ( ).get ( );
	}

	const DefaultSymbolType & getTerminatingSymbol ( ) const {
		return accessElement < TerminatingSymbol > ( ).get ( );
	}

	/**
	 * Prints XML representation of the tree to the output stream.
	 * @param out output stream to which print the tree
	 * @param tree tree to print
	 */
	virtual void operator >>( std::ostream & out ) const;

	friend std::ostream & operator <<( std::ostream & out, const SuffixTrieTerminatingSymbol & instance );

	virtual int compare ( const ObjectBase & other ) const {
		if ( ext::type_index ( typeid ( * this ) ) == ext::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return ext::type_index ( typeid ( * this ) ) - ext::type_index ( typeid ( other ) );
	}

	virtual int compare ( const SuffixTrieTerminatingSymbol & other ) const;

	virtual explicit operator std::string ( ) const;

	static const std::string & getXmlTagName() {
		static std::string xmlTagName = "SuffixTrieTerminatingSymbol";

		return xmlTagName;
	}

	static SuffixTrieTerminatingSymbol parse ( ext::deque < sax::Token >::iterator & input );

	static void compose ( ext::deque < sax::Token > & out, const SuffixTrieTerminatingSymbol & index );

	virtual alib::ObjectBase * inc ( ) &&;

	typedef SuffixTrieTerminatingSymbol normalized_type;
};

} /* namespace tree */

namespace alib {

template < >
class ComponentConstraint< indexes::SuffixTrieTerminatingSymbol, DefaultSymbolType, indexes::GeneralAlphabet > {
public:
	static bool used ( const indexes::SuffixTrieTerminatingSymbol & index, const DefaultSymbolType & symbol ) {
		return index.getTerminatingSymbol ( ) == symbol || index.getRoot ( ).testSymbol ( symbol );
	}

	static bool available ( const indexes::SuffixTrieTerminatingSymbol &, const DefaultSymbolType & ) {
		return true;
	}

	static void valid ( const indexes::SuffixTrieTerminatingSymbol &, const DefaultSymbolType & ) {
	}
};

template < >
class ElementConstraint< indexes::SuffixTrieTerminatingSymbol, DefaultSymbolType, indexes::TerminatingSymbol > {
public:
	static bool available ( const indexes::SuffixTrieTerminatingSymbol & index, const DefaultSymbolType & symbol ) {
		return index.getAlphabet ( ).count ( symbol );
	}

	static void valid ( const indexes::SuffixTrieTerminatingSymbol &, const DefaultSymbolType & ) {
	}
};

} /* namespace alib */

#endif /* SUFFIX_TRIE_TERMINATING_SYMBOL_H_ */
